/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef COLORACTION_H
#define COLORACTION_H

#include "tkaction.h"
#include <kpopupmenu.h>

#include <qlayout.h>
#include <qcolor.h>
#include <qdict.h>

#include <qdom.h>

class ColorPanel;
class KivioToolBarButton;

class ColorPopupMenu : public KPopupMenu
{ Q_OBJECT
public:
  ColorPopupMenu( QWidget* parent = 0, const char* name = 0 );
  ~ColorPopupMenu();

public slots:
  void updateItemSize();
};
/****************************************************************************************/
class SelectColorAction : public TKAction
{ Q_OBJECT
public:
  enum Type {
    TextColor,
    LineColor,
    FillColor,
    Color
  };

  SelectColorAction( const QString& text, Type type, QObject* parent, const char* name );

  virtual ~SelectColorAction();

  QColor color() const { return m_pCurrentColor; }

  KPopupMenu* popupMenu() { return m_pMenu; }


public slots:
  void setCurrentColor( const QColor& );
  void setActiveColor( const QColor& );

signals:
  void colorSelected( const QColor& );

protected slots:
  void selectColorDialog();
  void panelColorSelected( const QColor& );
  void panelReject();

protected:
  virtual void initToolBarButton(TKToolBarButton*);
  void updatePixmap();
  void updatePixmap(TKToolBarButton*);

protected:
  ColorPopupMenu* m_pMenu;
  ColorPanel* m_pStandartColor;
  ColorPanel* m_pRecentColor;
  int m_type;

  QColor m_pCurrentColor;
  QRect iconColorRect;
};
/****************************************************************************************/
class ColorPanelButton : public QFrame
{ Q_OBJECT
public:
  ColorPanelButton( const QColor&, QWidget* parent, const char* name = 0 );
  ~ColorPanelButton();

  void setActive( bool );

  QColor panelColor() { return m_Color; }

signals:
  void selected( const QColor& );

protected:
  virtual void paintEvent( QPaintEvent* );
  virtual void enterEvent( QEvent* );
  virtual void leaveEvent( QEvent* );
  virtual void mouseReleaseEvent( QMouseEvent* );

  QColor m_Color;
  bool m_bActive;
};
/****************************************************************************************/
class ColorPanel : public QWidget
{ Q_OBJECT

public:
  ColorPanel( QWidget* parent = 0L, const char* name = 0 );
  ~ColorPanel();

  void setActiveColor( const QColor& );
  void setNumCols( int col );
  void clear();


public slots:
  void insertColor( const QColor& );
  void insertColor( const QColor&, const QString& );
  void selected( const QColor& );

signals:
  void colorSelected( const QColor& );
  void reject();
  void sizeChanged();

protected:
  void addToGrid( ColorPanelButton* );
  void resetGrid();

  virtual void mouseReleaseEvent( QMouseEvent* );

  QGridLayout* m_pLayout;
  int m_iWidth;
  int m_iX;
  int m_iY;

  QColor m_activeColor;
  QDict<ColorPanelButton> m_pColorDict;
};

#endif

