/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>

#include <qpainter.h>
#include <qdrawutil.h>
#include <qkeycode.h>
#include <qregexp.h>
#include <qpoint.h>
#include <qprinter.h>
#include <qcursor.h>
#include <qstack.h>
#include <qbuffer.h>
#include <qmessagebox.h>
#include <qclipboard.h>
#include <qpicture.h>
#include <qdom.h>
#include <qtextstream.h>
#include <qdragobject.h>
#include <qmime.h>

#include <klocale.h>
#include <koPageLayoutDia.h>
#include <kglobal.h>

#include "kivio_page.h"
#include "kivio_map.h"
#include "kivio_doc.h"
#include "kivio_canvas.h"

#include "kivio_common.h"
#include "kivio_group_stencil.h"
#include "kivio_layer.h"
#include "kivio_painter.h"
#include "kivio_point.h"
#include "kivio_ps_printer.h"
#include "kivio_stencil.h"


#include <koStream.h>

#include <strstream.h>

int KivioPage::s_id = 0L;
QIntDict<KivioPage>* KivioPage::s_mapPages;

KivioPage::KivioPage( KivioMap *_map, const char *_name )
: QObject( _map, _name ),
  m_pCurLayer(NULL)
{
  if ( s_mapPages == 0L )
    s_mapPages = new QIntDict<KivioPage>;
  m_id = s_id++;
  s_mapPages->insert( m_id, this );

  m_pMap = _map;
  m_pDoc = _map->doc();

  // Make sure the layers auto-delete themselves
  m_pCurLayer = new KivioLayer(this);
  m_lstLayers.append( m_pCurLayer );
  m_lstLayers.setAutoDelete(true);

  m_lstSelection.setAutoDelete(false);

  m_strName = _name;

  setHidden(false);
  // Get a unique name so that we can offer scripting
  if ( !_name ) {
    QCString s;
    s.sprintf("Page%i", s_id );
    setName( s.data() );
  }
  m_pPageLayout = KoPageLayoutDia::standardLayout();
}

KivioPage::~KivioPage()
{
  debug("AHHHHH PAGE DYING!");
  s_mapPages->remove(m_id);
}

KivioPage* KivioPage::find( int _id )
{
  if ( !s_mapPages )
    return 0L;

  return (*s_mapPages)[ _id ];
}

void KivioPage::print( KivioPSPrinter *printer )
{
    KivioLayer *pLayer = m_lstLayers.first();
    while( pLayer )
    {
        if( pLayer->visible() )
        {
            pLayer->printContent( *printer );
            pLayer = m_lstLayers.next();
        }
    }
}

void KivioPage::print( QPainter &painter, QPrinter *_printer )
{
}

void KivioPage::printPage( QPainter &_painter, const QRect& page_range, const QRect& view )
{
}

QDomElement KivioPage::save( QDomDocument& doc )
{
    debug("+SAVE KivioPage");

    // Write the name and 'hide' flag first as attributes
    QDomElement page = doc.createElement( "page" );
    page.setAttribute( "name", m_strName );
    page.setAttribute( "hide", (int)m_bPageHide );

    // Create a child element for the page layout
    QDomElement layoutE = saveLayout( doc );
    page.appendChild( layoutE );

    // Iterate through all layers saving them as child elements
    KivioLayer *pLayer = m_lstLayers.first();
    while( pLayer )
    {
        debug("+SAVE calling save layer");
        QDomElement layerE = pLayer->saveXML(doc);
        if( layerE.isNull() )
        {
            debug("********** HOLY SHIT ***********");
        }

        page.appendChild( layerE );

        pLayer = m_lstLayers.next();
    }

    return page;
}

QDomElement KivioPage::saveLayout( QDomDocument &doc )
{
    QDomElement e = doc.createElement("PageLayout");

    XmlWriteInt( e, "format", m_pPageLayout.format );
    XmlWriteInt( e, "orientation", m_pPageLayout.orientation );
    XmlWriteInt( e, "unit", m_pPageLayout.unit );

    XmlWriteDouble( e, "ptWidth", m_pPageLayout.ptWidth );
    XmlWriteDouble( e, "ptHeight", m_pPageLayout.ptHeight );
    XmlWriteDouble( e, "ptLeft", m_pPageLayout.ptLeft );
    XmlWriteDouble( e, "ptRight", m_pPageLayout.ptRight );
    XmlWriteDouble( e, "ptTop", m_pPageLayout.ptTop );
    XmlWriteDouble( e, "ptBottom", m_pPageLayout.ptBottom );
    XmlWriteDouble( e, "mmWidth", m_pPageLayout.mmWidth );
    XmlWriteDouble( e, "mmHeight", m_pPageLayout.mmHeight );
    XmlWriteDouble( e, "mmLeft", m_pPageLayout.mmLeft );
    XmlWriteDouble( e, "mmTop", m_pPageLayout.mmTop );
    XmlWriteDouble( e, "mmRight", m_pPageLayout.mmRight );
    XmlWriteDouble( e, "mmBottom", m_pPageLayout.mmBottom );
    XmlWriteDouble( e, "inchWidth", m_pPageLayout.inchWidth );
    XmlWriteDouble( e, "inchHeight", m_pPageLayout.inchHeight );
    XmlWriteDouble( e, "inchLeft", m_pPageLayout.inchLeft );
    XmlWriteDouble( e, "inchTop", m_pPageLayout.inchTop );
    XmlWriteDouble( e, "inchRight", m_pPageLayout.inchRight );
    XmlWriteDouble( e, "inchBottom", m_pPageLayout.inchBottom );

    return e;
}

bool KivioPage::loadLayout( const QDomElement &e )
{
    m_pPageLayout.format = (KoFormat)XmlReadInt( e, "format", PG_US_LETTER );
    m_pPageLayout.orientation = (KoOrientation)XmlReadInt( e, "orientation", PG_PORTRAIT );
    m_pPageLayout.unit = (KoUnit)XmlReadInt( e, "unit", PG_INCH );

    m_pPageLayout.ptWidth = XmlReadDouble( e, "ptWidth", 8.5 * 72.0 );
    m_pPageLayout.ptHeight = XmlReadDouble( e, "ptHeight", 11.0 * 72.0 );
    m_pPageLayout.ptLeft = XmlReadDouble( e, "ptLeft", 1.0 );
    m_pPageLayout.ptRight = XmlReadDouble( e, "ptRight", 1.0 );
    m_pPageLayout.ptTop = XmlReadDouble( e, "ptTop", 1.0 );
    m_pPageLayout.ptBottom = XmlReadDouble( e, "ptBottom", 1.0 );
    m_pPageLayout.mmWidth = XmlReadDouble( e, "mmWidth", 1.0 );
    m_pPageLayout.mmHeight = XmlReadDouble( e, "mmHeight", 1.0 );
    m_pPageLayout.mmLeft = XmlReadDouble( e, "mmLeft", 1.0 );
    m_pPageLayout.mmTop = XmlReadDouble( e, "mmTop", 1.0 );
    m_pPageLayout.mmRight = XmlReadDouble( e, "mmRight", 1.0 );
    m_pPageLayout.mmBottom = XmlReadDouble( e, "mmBottom", 1.0 );
    m_pPageLayout.inchWidth = XmlReadDouble( e, "inchWidth", 1.0 );
    m_pPageLayout.inchHeight = XmlReadDouble( e, "inchHeight", 1.0 );
    m_pPageLayout.inchLeft = XmlReadDouble( e, "inchLeft", 1.0 );
    m_pPageLayout.inchTop = XmlReadDouble( e, "inchTop", 1.0 );
    m_pPageLayout.inchRight = XmlReadDouble( e, "inchRight", 1.0 );
    m_pPageLayout.inchBottom = XmlReadDouble( e, "inchBottom", 1.0 );

    return true;
}

bool KivioPage::isLoading()
{
  return m_pDoc->isLoading();
}

bool KivioPage::loadXML( const QDomElement& pageE )
{
    m_strName = pageE.attribute("name");
    if (m_strName.isEmpty())
        return false;

    debug("-LOAD KivioPage");

	m_bPageHide = (int)pageE.attribute("hide").toInt();
	
	// Clear the layer list
	KivioLayer *pLayer;
	m_lstLayers.clear();
	
	QDomNode node = pageE.firstChild();
	while( !node.isNull() )
	{
	    if( node.nodeName() == "KivioLayer" )
	    {
	        debug("-LOAD About to load kivio layer");
	        pLayer = new KivioLayer(this);
	        if( pLayer->loadXML( node.toElement() )==false )
	        {
	            delete pLayer;
	            pLayer = NULL;
	        }
	        else
	        {
	            m_lstLayers.append( pLayer );
	            pLayer = NULL;
	        }
	    }
	    else if( node.nodeName() == "PageLayout" )
	    {
	        loadLayout( node.toElement() );
	    }
	    else
	    {
	        debug("-LOAD KivioLayer::loadXML() - unknown node found, %s", node.nodeName().ascii());
	    }
	
	    node = node.nextSibling();
	}
	
	m_pCurLayer = m_lstLayers.first();
	if( !m_pCurLayer )
	{
	    debug("KivioLayer::loadXML() - No layers loaded!! BIGGGGG PROBLEMS!");
	}
	
	// Now that we are done loading, fix all the connections
	KivioLayer *pLayerBak;
	
	pLayer = m_lstLayers.first();
	while( pLayer )
	{
	    pLayerBak = pLayer;
	        
	    debug("KivioLayer::loadXML() - loading layer connections");
	    pLayer->searchForConnections(this);
	    
	    m_lstLayers.find( pLayerBak );
	
	    pLayer = m_lstLayers.next();
	}

  return true;
}

void KivioPage::update()
{
}

KivioPage* KivioPage::findPage( const QString& name )
{
  if (!m_pMap)
    return 0L;

  return m_pMap->findPage(name);
}

bool KivioPage::setPageName( const QString& name, bool init )
{
  if ( map()->findPage( name ) )
    return false;

  if ( m_strName == name )
    return true;

  QString old_name = m_strName;
  m_strName = name;

  if (init)
    return true;

  emit sig_nameChanged( this, old_name );
  emit sig_updateView(this);

  return true;
}

void KivioPage::paperLayoutDlg()
{
  if ( KoPageLayoutDia::pageLayout(m_pPageLayout,m_pHeadFoot,FORMAT_AND_BORDERS|HEADER_AND_FOOTER) ) {
    emit sig_updateView(this);
  }
}

void KivioPage::paintContent( KivioPainter& painter, const QRect& rect, bool transparent, QPoint p0, int zoom )
{
    KivioLayer *pLayer = m_lstLayers.first();
    while( pLayer )
    {
        if( pLayer->visible() )
        {
            pLayer->paintContent( painter, rect, transparent, p0, zoom );
            pLayer = m_lstLayers.next();
        }
    }



    // Now the second iteration - connection targets
    m_pCurLayer->paintConnectorTargets( painter, rect, transparent, p0, zoom );
    /*
    * FIXME: Make this only draw connection targets of those layers marked
    * as Connectable
    */
    /*
    pLayer = m_lstLayers.first();
    while( pLayer )
    {
        pLayer->paintConnectorTargets( painter, rect, transparent, p0, zoom );

        pLayer = m_lstLayers.next();
    }
    */

    // Now the third iteration - selection handles
    m_pCurLayer->paintSelectionHandles( painter, rect, transparent, p0, zoom );
    /*
    pLayer = m_lstLayers.first();
    while( pLayer )
    {
        pLayer->paintSelectionHandles( painter, rect, transparent, p0, zoom );

        pLayer = m_lstLayers.next();
    }
    */

}

bool KivioPage::addStencil( KivioStencil *pStencil )
{
    if( !pStencil )
    {
        debug("KivioPage::addStencil() - Null stencil passed");
        return false;
    }

    if( !m_pCurLayer )
    {
        debug("KivioPage::addStencil() - NULL current layer");
        return false;
    }

    return m_pCurLayer->addStencil( pStencil );
}

void KivioPage::selectStencils( float x, float y, float w, float h )
{
    // Iterate through all stencils of this layer
    KivioStencil *pStencil = m_pCurLayer->stencilList()->first();
    while( pStencil )
    {
        // Is it in the rectangle?
        if( stencilInRect( x, y, w, h, pStencil )==true )
        {
            selectStencil( pStencil );
        }

        pStencil = m_pCurLayer->stencilList()->next();
    }


    /*
    * No multi-layer selections
    */
    /*
    KivioLayer *pLayer = m_lstLayers.first();
    while( pLayer )
    {
        if( pLayer->visible() == true )
        {
            // Iterate through all stencils of this layer
            KivioStencil *pStencil = pLayer->stencilList()->first();
            while( pStencil )
            {
                // Is it in the rectangle?
                if( stencilInRect( x, y, w, h, pStencil )==true )
                {
                    selectStencil( pStencil );
                }

                pStencil = pLayer->stencilList()->next();
            }
        }

        pLayer = m_lstLayers.next();
    }
    */
}

bool KivioPage::stencilInRect( float x, float y, float w, float h, KivioStencil *pStencil )
{
    float sx, sy, sw, sh;

    sx = pStencil->x();
    sy = pStencil->y();
    sw = pStencil->w();
    sh = pStencil->h();

    if( sx >= x &&
        sy >= y &&
        sx+sw <= x+w &&
        sy+sh <= y+h )
        return true;

    return false;
}


void KivioPage::selectStencil( KivioStencil *pStencil )
{
    if( !pStencil )
    {
        debug("KivioPage::selectStencil - AHHHH! NULL STENCIL!");
        return;
    }

    // Don't allow reselection
    if( m_lstSelection.findRef( pStencil ) != -1 )
        return;

    debug("KivioPage::selectStencil - Selecting stencil");
    pStencil->select();
    m_lstSelection.append( pStencil );

    m_pDoc->slotSelectionChanged();
}

bool KivioPage::unselectStencil( KivioStencil *pStencil )
{
    pStencil->unselect();
    m_pDoc->slotSelectionChanged();
    return m_lstSelection.removeRef( pStencil );
}

void KivioPage::selectAllStencils()
{
    unselectAllStencils();

    KivioStencil* pStencil = m_pCurLayer->stencilList()->first();
    while( pStencil )
    {
        pStencil->select();
        m_lstSelection.append(pStencil);

        pStencil = m_pCurLayer->stencilList()->next();
    }


    /*
     * The following code is commented out because selections must
     * remain in the current layer.
    */
/*
    KivioLayer* pLayer = m_lstLayers.first();
    while( pLayer )
    {
        if( pLayer->visible() )
        {
            pStencil = pLayer->stencilList()->first();
            while( pStencil )
            {
                pStencil->select();
                m_lstSelection.append(pStencil);

                pStencil = pLayer->stencilList()->next();
            }
        }

        pLayer = m_lstLayers.next();
    }
*/
    m_pDoc->slotSelectionChanged();
}

void KivioPage::unselectAllStencils()
{
    KivioStencil *pStencil;

    pStencil = m_lstSelection.first();
    while( pStencil )
    {
        pStencil->unselect();

        pStencil = m_lstSelection.next();
    }

    m_lstSelection.clear();

    m_pDoc->slotSelectionChanged();
}

bool KivioPage::isStencilSelected( KivioStencil *pStencil )
{
    return m_lstSelection.findRef( pStencil )==-1 ? false : true;
}

/**
 * Looks for stencils at a given point in the canvas
 *
 * @param pPoint The point to search with
 *
 * This will iterate through all visible layers and check
 * for stencils.  The first one it finds will be returned.
 * If none are found, it will return NULL.
 */
KivioStencil *KivioPage::checkForStencil( KivioPoint *pPoint, int *collisionType )
{
    KivioStencil *pStencil;
    int colType;


    /*
     * This code is commented out because selecting a stencil should only take place
     * on the current layer.  The following code searches all layers.
    */
    // Start with the last layer since it is the top
    /*
    KivioLayer* pLayer = m_lstLayers.last();
    while( pLayer )
    {
        pStencil = pLayer->checkForStencil( pPoint, &colType );
        if( pStencil )
        {
            *collisionType = colType;
            return pStencil;
        }

        pLayer = m_lstLayers.prev();
    }
    */

    pStencil = m_pCurLayer->checkForStencil( pPoint, &colType );
    if( pStencil )
    {
        *collisionType = colType;
        return pStencil;
    }


    *collisionType = kctNone;
    return NULL;
}

void KivioPage::deleteSelectedStencils()
{
    KivioStencil *pStencil;

    // Iterate through all items in the selection list
    m_lstSelection.first();
    pStencil = m_lstSelection.take();
    while( pStencil )
    {
        /*
         * The following code is commented out because selections can only take
         * place on the current layer.
         */
        // Now iterate through the layers attempting to delete the current stencil.  If
        // true is returned, that means that the delete was successful and we can stop
        // looking through layers.
        /*
        KivioLayer* pLayer = m_lstLayers.first();
        while( pLayer )
        {
            if( pLayer->removeStencil( pStencil )==true )
            {
                break;
            }

            pLayer = m_lstLayers.next();
        }
        */

        if( m_pCurLayer->removeStencil( pStencil ) == false )
        {
            debug("KivioPage::deleteSelectedStencils() - Failed to locate a selected stencil in the current layer");
        }

        pStencil = m_lstSelection.take();
    }
}

void KivioPage::groupSelectedStencils()
{
    KivioGroupStencil *pGroup;
    KivioStencil *pTake;
    KivioStencil *pStencil;


    debug("*GROUP* About to group");
    // Can't group 0 or 1 stencils
    if( m_lstSelection.count() <= 1 )
        return;

    pGroup = new KivioGroupStencil();

    // Iterate through all items in the selection list, taking them from the layer, then adding
    // them to the group

    pStencil = m_lstSelection.first();
    while( pStencil )
    {

        debug("*GROUP* Duplicating 1");
        
        // Take the stencil out of it's layer
        pTake = m_pCurLayer->takeStencil( pStencil );
        if( !pTake )
        {
            debug("KivioPage::groupSelectedStencil() - Failed to take() one of the selected stencils. CRAP!");
        }
        else
        {
            // Add it to the group
            pGroup->addToGroup( pTake );
        }

        pStencil = m_lstSelection.next();
    }

    // Unselect the old ones
    unselectAllStencils();


    // Add the group as the selected stencil
    m_pCurLayer->addStencil( pGroup );

    selectStencil( pGroup );
}

// The following is the old implementation of groupSelectedStencils.  It did
// not preserve connections so a new method was devised.
/*
void KivioPage::groupSelectedStencils()
{
    KivioGroupStencil *pGroup;
    KivioStencil *pTake;
    KivioStencil *pStencil;


    debug("*GROUP* About to group");
    // Can't group 0 or 1 stencils
    if( m_lstSelection.count() <= 1 )
        return;

    pGroup = new KivioGroupStencil();

    // Iterate through all items in the selection list, duplicating them, then adding
    // them to the group

    pStencil = m_lstSelection.first();
    while( pStencil )
    {

        debug("*GROUP* Duplicating 1");
        
        // Dup the stencil & group it
        pTake  = pStencil->duplicate();

        pGroup->addToGroup( pDuplicate );

        pStencil = m_lstSelection.next();
    }

    // Kill the old selections
    deleteSelectedStencils();


    // Add the group as the selected stencil
    m_pCurLayer->addStencil( pGroup );

    selectStencil( pGroup );
}
*/
void KivioPage::ungroupSelectedStencils()
{
    KivioStencil *pSelStencil, *pStencil;
    QList<KivioStencil> *pList;
    QList<KivioStencil> *pSelectThese = new QList<KivioStencil>;

    pSelectThese->setAutoDelete(false);

    // Iterate through all selected stencils
    pSelStencil = m_lstSelection.first();
    while( pSelStencil )
    {
        // If there is a list, it is a group stencil
        pList = pSelStencil->groupList();
        if( pList )
        {
            pList->first();
            pStencil = pList->take();
            while( pStencil )
            {
                addStencil( pStencil );

                pSelectThese->append( pStencil );

                pStencil = pList->take();
            }

            /*
             *  The following is commented out because the group should be on the
             * current layer since selections must be on the current layer.
             */
            // Since this was a group, it is now an empty stencil, so we remove it
            // from the selection list, and then remove it from the layer it came
            // from, but we have to search for that.
            /*
            pSelStencil = m_lstSelection.take();
            KivioLayer *pLayer = m_lstLayers.first();
            while( pLayer )
            {
                if( pLayer->removeStencil( pSelStencil )==true )
                    break;

                pLayer = m_lstLayers.next();
            }
            */
            
            // Remove the current stencil from the selection list(the group we just disassembled)
            m_lstSelection.take();

            // Remove it permanently from the layer
            if( m_pCurLayer->removeStencil( pSelStencil )==false )
            {
                debug("KivioPage::ungroupSelectedStencil() - Failed to locate the group shell for deletion");
            }
        }


        pSelStencil = m_lstSelection.next();
    }

    // Now iterate through the selectThese list and select
    // those stencils
    pStencil = pSelectThese->first();
    while( pStencil )
    {
        selectStencil( pStencil );

        pStencil = pSelectThese->next();
    }



    delete pSelectThese;
}

bool KivioPage::pagePropertiesDlg()
{
    return KoPageLayoutDia::pageLayout( m_pPageLayout, m_pHeadFoot, 1 );
}

void KivioPage::bringToFront()
{
    KivioStencil *pStencil, *pMove;
    KivioLayer *pLayer;

    QList <KivioStencil> newList;

    pLayer = m_pCurLayer;

    newList.setAutoDelete(false);

    /*
     * We iterate through all stencils since order must be maintained
     * amongst the selection during the move.
     */
    pStencil = pLayer->firstStencil();
    while( pStencil )
    {
        if( isStencilSelected( pStencil )==true )
        {
            pMove = pLayer->takeStencil();
            if( pMove )
            {
                newList.append(pMove);
                pStencil = pLayer->currentStencil();
            }
            else  // In the case of error, the outside else won't execute
            {
                pStencil = pLayer->nextStencil();
            }
        }
        else
        {
            pStencil = pLayer->nextStencil();
        }
    }

    // push them back in, in reverse order
    pStencil = newList.first();
    while( pStencil )
    {
        pLayer->stencilList()->append(pStencil);

        pStencil = newList.next();
    }
}

void KivioPage::sendToBack()
{
    KivioStencil *pStencil, *pMove;
    KivioLayer *pLayer;

    QList <KivioStencil> newList;

    pLayer = m_pCurLayer;

    newList.setAutoDelete(false);

    /*
     * We iterate through all stencils since order must be maintained
     * amongst the selection during the move.
     */
    pStencil = pLayer->firstStencil();
    while( pStencil )
    {
        if( isStencilSelected( pStencil )==true )
        {
            pMove = pLayer->takeStencil();
            if( pMove )
            {
                newList.append(pMove);
                pStencil = pLayer->currentStencil();
            }
            else  // In the case of error, the outside else won't execute
            {
                pStencil = pLayer->nextStencil();
            }
        }
        else
        {
            pStencil = pLayer->nextStencil();
        }
    }

    // push them back in, in reverse order
    pStencil = newList.last();
    while( pStencil )
    {
        pLayer->stencilList()->insert(0, pStencil);

        pStencil = newList.prev();
    }
}

void KivioPage::copy()
{
    KivioGroupStencil *pGroup = new KivioGroupStencil();
    KivioStencil *pStencil;
    KivioLayer *pLayer;

    if( m_lstSelection.count() <= 0 )
        return;

    debug("Copy");
    pLayer = m_pCurLayer;

    pStencil = pLayer->firstStencil();
    while( pStencil )
    {
        if( isStencilSelected( pStencil )==true )
        {
            pGroup->addToGroup( pStencil->duplicate() );
        }

        pStencil = pLayer->nextStencil();
    }

    m_pDoc->setClipboard( pGroup );
}

void KivioPage::cut()
{
    KivioGroupStencil *pGroup = new KivioGroupStencil();
    KivioStencil *pStencil;
    KivioLayer *pLayer;

    if( m_lstSelection.count() <= 0 )
        return;

    debug("Cut");
    pLayer = m_pCurLayer;

    pStencil = pLayer->firstStencil();
    while( pStencil )
    {
        if( isStencilSelected( pStencil )==true )
        {
            pGroup->addToGroup( pStencil->duplicate() );
        }

        pStencil = pLayer->nextStencil();
    }

    deleteSelectedStencils();

    m_pDoc->setClipboard( pGroup );
}

void KivioPage::paste(  )
{
    KivioStencil *pGroup;
    KivioStencil *pStencil, *pDup;
    QList<KivioStencil> *pList;
    QList<KivioStencil> *pSelectThese = new QList<KivioStencil>;

    pSelectThese->setAutoDelete(false);

    pGroup = m_pDoc->clipboard();
    if( !pGroup )
    {
        delete pSelectThese;
        return;
    }

    debug("Paste");

    // If there is a list, it is a group stencil
    pList = pGroup->groupList();
    if( pList )
    {
        pStencil = pList->first();
        while( pStencil )
        {
            pDup = pStencil->duplicate();

            // FIXME: Make this offset configurable
            pDup->setPosition( pDup->x() + 10.0f, pDup->y() + 10.0f );
            
            
            addStencil( pDup );

            pSelectThese->append( pDup );

            pStencil = pList->next();
        }
    }

    unselectAllStencils();

    // Now iterate through the selectThese list and select
    // those stencils
    pStencil = pSelectThese->first();
    while( pStencil )
    {
        selectStencil( pStencil );

        pStencil = pSelectThese->next();
    }


    delete pSelectThese;
}

int KivioPage::generateStencilIds( int next )
{
    KivioLayer *pLayer;
    
    pLayer = m_lstLayers.first();
    while( pLayer )
    {
    
        next = pLayer->generateStencilIds( next );
        
        pLayer = m_lstLayers.next();
    }
    
    return next;
}

KivioLayer *KivioPage::firstLayer()
{
    return m_lstLayers.first();
}

KivioLayer *KivioPage::nextLayer()
{
    return m_lstLayers.next();
}