/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "straight_connector.h"

#include "kivio_common.h"
#include "kivio_connector_target.h"
#include "kivio_layer.h"
#include "kivio_page.h"
#include "kivio_painter.h"
#include "kivio_stencil.h"
#include "kivio_stencil_spawner.h"
#include "kivio_stencil_spawner_info.h"
#include "kivio_stencil_spawner_set.h"
#include "kivio_page.h"

#include <qcolor.h>
#include <qpixmap.h>

#include "straight_connector.xpm"


static QPixmap *pIcon = NULL;

static KivioStencilSpawnerInfo sinfo = KivioStencilSpawnerInfo("Dave Marotti", "Straight Connector", "Basic Straight Connector", "0.1", "http://localhost/", "", "off");

extern "C" {
    KivioStencil *NewStencil();
    QPixmap *GetIcon();
    KivioStencilSpawnerInfo *GetSpawnerInfo();
}

KivioStencil *NewStencil()
{
    return new KivioStraightConnector();
}


//FIXME: Is this a memory leak?
QPixmap *GetIcon()
{
    if( pIcon )
        return pIcon;

    pIcon = new QPixmap( (const char **)straight_connector_xpm );
    return pIcon;
}

KivioStencilSpawnerInfo *GetSpawnerInfo()
{
    return &sinfo;
}


KivioStraightConnector::KivioStraightConnector()
{
    m_start.setPosition(0.0f, 0.0f);
    m_start.setStencil(this);
    m_end.setPosition(72.0f, 72.0f);
    m_end.setStencil(this);

    m_lineWidth = 1.0f;
    m_fgColor = QColor(0,0,0);
}

KivioStencil *KivioStraightConnector::duplicate()
{
    KivioStraightConnector *pStencil = new KivioStraightConnector();

    pStencil->setSpawner( m_pSpawner );

    pStencil->m_start.setPosition( m_start.x(), m_start.y() );
    pStencil->m_end.setPosition( m_end.x(), m_end.y() );
    pStencil->m_lineWidth = m_lineWidth;
    pStencil->m_fgColor = m_fgColor;

    return pStencil;
}

KivioStraightConnector::~KivioStraightConnector()
{

}

void KivioStraightConnector::setX( float x )
{
    float dx = x - m_x;

    m_x += dx;

    m_start.setX( m_start.x() + dx );
    m_end.setX( m_end.x() + dx );

    m_start.disconnect();
    m_end.disconnect();
}

void KivioStraightConnector::setY( float y )
{
    float dy = y - m_y;

    m_y += dy;

    m_start.setY( m_start.y() + dy );
    m_end.setY( m_end.y() + dy );

    m_start.disconnect();
    m_end.disconnect();
}

void KivioStraightConnector::setPosition( float x, float y )
{
    float dx = x - m_x;
    float dy = y - m_y;

    m_x += dx;
    m_y += dy;

    m_start.setPosition( m_start.x() + dx, m_start.y() + dy, false );
    m_end.setPosition( m_end.x() + dx, m_end.y() + dy, false );

    m_start.disconnect();
    m_end.disconnect();
}

/*
void KivioStraightConnector::setW( float w )
{
    m_end.setPosition( m_start.x() + w, m_end.y(), false );
    m_end.disconnect();
}

void KivioStraightConnector::setH( float h )
{
    m_end.setPosition( m_end.x(), m_start.y() + h, false );
    m_end.disconnect();
}

void KivioStraightConnector::setDimensions( float w, float h )
{
    m_end.setPosition( m_start.x() + w, m_start.y() + h, false );
    m_end.disconnect();
}
*/
void KivioStraightConnector::setStartPoint( float x, float y )
{
    m_start.setPosition( x, y, false );
    m_start.disconnect();
}

void KivioStraightConnector::setEndPoint( float x, float y )
{
    m_end.setPosition( x, y, false );
    m_end.disconnect();
}

void KivioStraightConnector::paint( KivioIntraStencilData *pData )
{
    KivioPainter *painter = pData->painter;
    float scale = pData->scale;
    float x1, y1, x2, y2;

    painter->setFGColor( m_fgColor );
    painter->setLineWidth( m_lineWidth );

    x1 = m_start.x() * scale;
    x2 = m_end.x() * scale;

    y1 = m_start.y() * scale;
    y2 = m_end.y() * scale;

    painter->drawLine( x1, y1, x2, y2 );
}

void KivioStraightConnector::paintOutline( KivioIntraStencilData *pData )
{
    KivioPainter *painter = pData->painter;
    float scale = pData->scale;
    float x1, y1, x2, y2;

    painter->setLineWidth( m_lineWidth );

    x1 = m_start.x() * scale;
    x2 = m_end.x() * scale;

    y1 = m_start.y() * scale;
    y2 = m_end.y() * scale;

    painter->drawLine( x1, y1, x2, y2 );
}

void KivioStraightConnector::paintConnectorTargets( KivioIntraStencilData * )
{
}

void KivioStraightConnector::paintSelectionHandles( KivioIntraStencilData *pData )
{
    // Handle Width
    const float HW = 6.0f;
    const float HWP1 = HW+1.0f;

    // Handle Width Over 2
    const float HWo2 = HW/2.0f;

    // Stencil data
    float scale = pData->scale;
    KivioPainter *painter = pData->painter;
    float x1, y1, x2, y2;

    // Calculate the end points
    x1 = m_start.x() * scale - HWo2;
    x2 = m_end.x() * scale - HWo2;

    y1 = m_start.y() * scale - HWo2;
    y2 = m_end.y() * scale - HWo2;


    painter->setFGColor( QColor(0,0,0) );
    painter->setLineWidth(1.0f);

    if( m_start.target() )
        painter->setBGColor( QColor( 200, 0, 0 ) );
    else
        painter->setBGColor( QColor(0,200,0) );
    painter->fillRect( x1, y1, HWP1, HWP1 );


    if( m_end.target() )
        painter->setBGColor( QColor( 200, 0, 0 ) );
    else
        painter->setBGColor( QColor(0,200,0) );
    painter->fillRect( x2, y2, HWP1, HWP1 );
}

KivioCollisionType KivioStraightConnector::checkForCollision( KivioPoint *p )
{
    const float thresh = 4.0f;

    // Check against control points
    float x1, y1, x2, y2;

    // Calculate the end points
    x1 = m_start.x();
    x2 = m_end.x();

    y1 = m_start.y();
    y2 = m_end.y();

    float _x = p->x();
    float _y = p->y();

    if( _x >= x1 - thresh &&
        _x <= x1 + thresh &&
        _y >= y1 - thresh &&
        _y <= y1 + thresh )
    {
        return (KivioCollisionType)(kctCustom + 1);
    }

    if( _x >= x2 - thresh &&
        _x <= x2 + thresh &&
        _y >= y2 - thresh &&
        _y <= y2 + thresh )
    {
        return (KivioCollisionType)(kctCustom + 2);
    }


    if( p->x() >= m_x &&
        p->x() < m_x + m_w &&
        p->y() >= m_y &&
        p->y() < m_y + m_h )
        return kctBody;


    return kctNone;
}

void KivioStraightConnector::customDrag( KivioCustomDragData *pData )
{
//    float dx = pData->dx;
//    float dy = pData->dy;
    float _x = pData->x;
    float _y = pData->y;
    int id = pData->id;


    KivioConnectorPoint *pPoint;

    switch( id )
    {
        case kctCustom + 1:
            pPoint = &m_start;
            break;

        case kctCustom + 2:
            pPoint = &m_end;
            break;

        default:
            debug("KivioStraightConnector::customDrag() - Unknown drag ID: %d", id );
            return;
    }

    pPoint->setPosition( _x, _y, true );

/*
    KivioPage *pPage = pData->page;
    KivioLayer *pLayer = pPage->layers()->first();
    while( pLayer )
    {
        if( pLayer->visible()==false )
        {
            pLayer = pPage->layers()->next();
            continue;
        }

        KivioStencil *pStencil = pLayer->stencilList()->last();
        while( pStencil )
        {
            if( pStencil->connectToTarget( pPoint, 8.0f ) )
            {
                return;
            }

            pStencil = pLayer->stencilList()->prev();
        }

        pLayer = pPage->layers()->next();
    }
*/
    KivioLayer *pLayer = pData->page->curLayer();

    KivioStencil *pStencil = pLayer->lastStencil();
    while( pStencil )
    {
        if( pStencil != this )
        {
            if( pStencil->connectToTarget( pPoint, 8.0f ) )
            {
                return;
            }
        }
        
        pStencil = pLayer->prevStencil();
    }


    // Nope, disconnect
    pPoint->disconnect();
}

void KivioStraightConnector::updateGeometry()
{
    float minX, minY, maxX, maxY;

    minX = m_start.x();
    if( minX > m_end.x() )
    {
        maxX = minX;
        minX = m_end.x();
    }
    else
    {
        maxX = m_end.x();
    }

    minY = m_start.y();
    if( minY > m_end.y() )
    {
        maxY = minY;
        minY = m_end.y();
    }
    else
    {
        maxY = m_end.y();
    }

    m_x = minX;
    m_y = minY;
    m_w = maxX - minX + 1.0f;
    m_h = maxY - minY + 1.0f;
}

QDomElement KivioStraightConnector::saveXML( QDomDocument &doc )
{
    debug("+SAVE KivioPluginStencil");
    QDomElement e = doc.createElement("KivioPluginStencil");

    XmlWriteString( e, "title", m_pSpawner->info()->title() );
    XmlWriteString( e, "setName", m_pSpawner->set()->name() );

    QDomElement conListE = doc.createElement("KivioConnectorList");
    
    QDomElement startE = m_start.saveXML( doc );
    QDomElement endE = m_end.saveXML( doc );
    
    conListE.appendChild( startE );
    conListE.appendChild( endE );
    
    e.appendChild( conListE );
    
    QDomElement foreE = doc.createElement("Foreground");
    XmlWriteFloat( foreE, "lineWidth", m_lineWidth );
    XmlWriteColor( foreE, "color", m_fgColor );
    e.appendChild( foreE );

    return e;
}

bool KivioStraightConnector::loadXML( const QDomElement &e )
{
    QDomNode node;
    QString nodeName;
    QDomElement nodeE;

    debug("-LOAD KivioStraightConnector");

    node = e.firstChild();
    while( !node.isNull() )
    {

        nodeName = node.nodeName();
        nodeE = node.toElement();

        if( nodeName == "KivioConnectorList" )
        {
            QDomNode tmpNode = nodeE.firstChild();
            
            if( tmpNode.isNull() )
            {
                break;
            }
            
            // Read the first node (start)
            QDomElement startE = tmpNode.toElement();
            m_start.loadXML( startE );
            
            tmpNode = tmpNode.nextSibling();
            if( tmpNode.isNull() )
                break;
                                                 
            QDomElement endE = tmpNode.toElement();
            
            m_end.loadXML( endE );
                
//            m_end.setPosition( XmlReadFloat( endE, "x", 1.0f ),
  //                             XmlReadFloat( endE, "y", 1.0f ), false );
            
        }
        else if( nodeName == "Foreground" )
        {
            m_lineWidth = XmlReadFloat( nodeE, "lineWidth", 1.0f );
            m_fgColor = XmlReadColor( nodeE, "color", QColor(0,0,0) );
        }

        node = node.nextSibling();
    }

    updateGeometry();

    return true;
}

void KivioStraightConnector::searchForConnections( KivioPage *pPage )
{
    bool startDone = false;
    bool endDone = false;

    debug("  KSC 1");
    
    if( m_start.targetId() == -1 )
        startDone = true;
        
    if( m_end.targetId() == -1 )
        endDone = true;

    // No connections? Bail!        
    if( startDone && endDone )
        return;

    debug("  KSC 2");
                
    KivioLayer *pLayer = pPage->firstLayer();
    while( pLayer && ( startDone==false || endDone==false ) )
    {
        KivioStencil *pStencil = pLayer->firstStencil();
        
        while( pStencil && ( startDone==false || endDone==false ) )
        {
            // No connecting to ourself!
            if( pStencil != this )
            {
                if( (!startDone) && 
                    (m_start.targetId() != -1) )
                {
                    if( pStencil->connectToTarget( &m_start, m_start.targetId() ) )
                    {
                        startDone = true;
                    }
                }
                
                if( (!endDone) &&
                    (m_end.targetId() != -1) )
                {
                    if( pStencil->connectToTarget( &m_end, m_end.targetId() ) )
                    {
                        endDone = true;
                    }
                }
            }
        
            pStencil = pLayer->nextStencil();
        }
        
        pLayer = pPage->nextLayer();
    }
}