/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef STRAIGHT_CONNECTOR_H
#define STRAIGHT_CONNECTOR_H

#include "kivio_connector_point.h"
#include "kivio_custom_drag_data.h"
#include "kivio_intra_stencil_data.h"
#include "kivio_point.h"
#include "kivio_stencil.h"

class KivioPage;

#include <qcolor.h>
#include <qdom.h>

class KivioStraightConnector : public KivioStencil
{
protected:
    KivioConnectorPoint m_start, m_end;
    float m_lineWidth;
    QColor m_fgColor;
    QString m_text;

public:
    KivioStraightConnector();
    virtual ~KivioStraightConnector();

    virtual void setFGColor( QColor f ) { m_fgColor=f; }
    virtual QColor fgColor() { return m_fgColor; }

    virtual void setLineWidth( float f ) { m_lineWidth = f; }
    virtual float lineWidth() { return m_lineWidth; }

    virtual void setX( float );
    virtual void setY( float );
    virtual void setPosition( float, float );

    virtual void setText( const QString &t ) { m_text = t; }
    virtual QString text() { return m_text; }

    virtual void setStartPoint( float, float );
    virtual void setEndPoint( float, float );

//    virtual void setW( float );
//    virtual void setH( float );
//    virtual void setDimensions( float, float );

    virtual void paint( KivioIntraStencilData * );
    virtual void paintOutline( KivioIntraStencilData * );
    virtual void paintConnectorTargets( KivioIntraStencilData * );
    virtual void paintSelectionHandles( KivioIntraStencilData * );

    virtual KivioCollisionType checkForCollision( KivioPoint * );
    virtual void customDrag( KivioCustomDragData * );

    virtual void updateGeometry();

    virtual bool loadXML( const QDomElement & );
    virtual QDomElement saveXML( QDomDocument & );

    virtual KivioStencil* duplicate();
    
    virtual void searchForConnections( KivioPage * );
    

};

#endif

