/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef TOOL_SELECT_H
#define TOOL_SELECT_H

#include "tool.h"
#include "tkunits.h"

class KivioView;
class KivioPage;
class QMouseEvent;

class KivioStencil;

class SelectTool : public Tool
{ Q_OBJECT
public:
  SelectTool( KivioView* view );
  ~SelectTool();

  virtual void processEvent( QEvent* );
  virtual void activate();
  virtual void deactivate();
  virtual void configure();

  void select(QRect);

signals:
  void operationDone();

protected:
    void mousePress(QMouseEvent *);
    void mouseMove(QMouseEvent *);
    void mouseRelease(QMouseEvent *);

    bool startResizing(QMouseEvent *);
    bool startDragging(QMouseEvent *);
    bool startCustomDragging(QMouseEvent *);
    bool startRubberBanding(QMouseEvent*);

    void continueDragging(QMouseEvent *);
    void continueCustomDragging(QMouseEvent *);
    void continueResizing(QMouseEvent *);
    void continueRubberBanding(QMouseEvent *);

    void endResizing(QMouseEvent *);
    void endDragging(QMouseEvent *);
    void endCustomDragging(QMouseEvent *);
    void endRubberBanding(QMouseEvent *);



    void changeMouseCursor( QMouseEvent * );
    int isOverResizeHandle( KivioStencil *pStencil, const float x, const float y );
    QPoint m_startPoint, m_releasePoint;
    TKPoint m_lastPoint;

    // Select Tool Mode
    enum {
        stmNone,
        stmDrawRubber,
        stmDragging,
        stmCustomDragging,
        stmResizing
    };
private:
    // Flag to indicate that we are drawing a rubber band
    int m_mode;
    KivioStencil *m_pResizingStencil;
    KivioStencil *m_pCustomDraggingStencil;
    int m_resizeHandle;
    char m_keys[32];
    bool m_shiftKey;
    int m_customDragID;

};

#endif


